/*
// UNIT TESTS OF THE BOT
                               
                               M1    M2    M3    LACT
INPUT: Test Command String = ~+0000,+0000,+0000,+0000
       Test Command String = ~+0255,+0255,+0255,+3200




OUTPUT: ir sensor array in cm : (fl, fr, l, r, bl, br) from front left to back right


--------- NEW TARGET Motor Values -----------
target_Motor1_Command: 0
target_Motor2_Command: 0
target_FlipperMotor_Command: 0
target_LACT_Command: 0
---------------------------------------
blip - 60 milliseconds elapsed --- ( 97, 68, 129, 204, 190, 83)

*/



//----- DistanceSaftey Defines -----
#define FORWARD_SAFE_DISTANCE     50
#define BACKWARD_SAFE_DISTANCE    50

//----- KeepAlive Variables -----
#define KEEP_ALIVE_TIMEOUT_1      10000 
#define KEEP_ALIVE_TIMEOUT_2      38 ////roughly every 2 seconds
unsigned long start, finished, elapsed;
int keepAlive_1 = KEEP_ALIVE_TIMEOUT_1;
int keepAlive_2 = KEEP_ALIVE_TIMEOUT_2;

//----- SerialComms Variables -----
char targetCommandString[] = {' ',' ',' ',' ',' '};
int target_Motor1_Command = 0;
int target_Motor2_Command = 0;
int target_FlipperMotor_Command = 0;
int target_LACT_Command = 0;

//----- IR BLOCK -----
#include <SharpIR.h>

#define IR_FRONT_LEFT         A5
#define IR_FRONT_RIGHT        A4
#define IR_LEFT               A3
#define IR_RIGHT              A2
#define IR_BACK_LEFT          A1
#define IR_BACK_RIGHT         A0

#define LONG_RANGE_IR_SENSOR  20150
#define MED_RANGE_IR_SENSOR   1080

SharpIR IR_FL(IR_FRONT_LEFT, LONG_RANGE_IR_SENSOR);
SharpIR IR_FR(IR_FRONT_RIGHT, LONG_RANGE_IR_SENSOR);
SharpIR IR_L(IR_LEFT, MED_RANGE_IR_SENSOR);
SharpIR IR_R(IR_RIGHT, MED_RANGE_IR_SENSOR);
SharpIR IR_BL(IR_BACK_LEFT, LONG_RANGE_IR_SENSOR);
SharpIR IR_BR(IR_BACK_RIGHT, LONG_RANGE_IR_SENSOR);
//--------------------


void setup() {
  // put your setup code here, to run once:
  Serial.begin( 250000 );

  Serial.println("iBot v1.0 by OrbitusRobotics");
  Serial.println("BEGIN STARTUP SEQUENCE");

  //---- IR BLOCK SETUP ----
  pinMode( IR_FRONT_LEFT, INPUT);
  pinMode( IR_FRONT_RIGHT, INPUT);
  pinMode( IR_LEFT, INPUT);
  pinMode( IR_RIGHT, INPUT);
  pinMode( IR_BACK_LEFT, INPUT);
  pinMode( IR_BACK_RIGHT, INPUT);
  //------------------------

  Serial.println("STARTUP SEQUENCE COMPLETE");
  Serial.println("iBot v1.0 ONLINE");
 
  start=millis();

}

void loop() {
  // put your main code here, to run repeatedly:
    keepAlive_SAFTEYSTOP();
    
    if(Serial.available() > 0) {
 
      Serial.print("Parsing Serial and resetting keepAlive\n");
      resetKeepAlive();
      
      byte b =  Serial.read();
      
      if (b == 126)
      {
        char charIn = 0;
  
        targetCommandString[0] = Serial.read();
        targetCommandString[1] = Serial.read();
        targetCommandString[2] = Serial.read();
        targetCommandString[3] = Serial.read();
        targetCommandString[4] = Serial.read();

        Serial.read(); //READ THE COMMA SEPARATING THINGS!!!
        //Serial.print("TargetCommandString M1:");
        //Serial.print( targetCommandString );
        target_Motor1_Command = atoi(targetCommandString);
  
        targetCommandString[0] = Serial.read();
        targetCommandString[1] = Serial.read();
        targetCommandString[2] = Serial.read();
        targetCommandString[3] = Serial.read();
        targetCommandString[4] = Serial.read();

        Serial.read(); //READ THE COMMA SEPARATING THINGS!!!
        //Serial.print("TargetCommandString M2:");
        //Serial.print( targetCommandString );
        target_Motor2_Command = atoi(targetCommandString);
        
        targetCommandString[0] = Serial.read();
        targetCommandString[1] = Serial.read();
        targetCommandString[2] = Serial.read();
        targetCommandString[3] = Serial.read();
        targetCommandString[4] = Serial.read();

        Serial.read(); //READ THE COMMA SEPARATING THINGS!!!
        //Serial.print("TargetCommandString FLipper:");
        //Serial.print( targetCommandString );
        target_FlipperMotor_Command = atoi(targetCommandString);
  
        targetCommandString[0] = Serial.read();
        targetCommandString[1] = Serial.read();
        targetCommandString[2] = Serial.read();
        targetCommandString[3] = Serial.read();
        targetCommandString[4] = Serial.read();
  
        Serial.read(); //READ THE COMMA SEPARATING THINGS!!!
        //Serial.print("TargetCommandString LACT:");
        //Serial.print( targetCommandString );
        target_LACT_Command = atoi(targetCommandString);

        
  
        setMotorCommand(target_Motor1_Command, target_Motor2_Command, target_FlipperMotor_Command, target_LACT_Command);
        
        //irReadOut();
        
        Serial.println("---------------------------------------");
        
      }
   }
}


int irReadOut_SAFTEYSTOP()
{
  /* Check for obstacles 
  Serial.print( "( " );
  Serial.print( IR_FL.distance() );
  Serial.print( ", " );
  Serial.print( IR_FR.distance() );
  Serial.print( ", " );
  Serial.print( IR_L.distance() );
  Serial.print( ", " );
  Serial.print( IR_R.distance() );
  Serial.print( ", " );
  Serial.print( IR_BL.distance() );
  Serial.print( ", " );
  Serial.print( IR_BR.distance() );
  Serial.print( ")\n" );
  */
  //FORWARD SAFTEY CHECKS
  if ( IR_FL.distance() < FORWARD_SAFE_DISTANCE )
    Serial.println( "SAFTEY SHUTDOWN ERROR! OBSTACLE IS BLOCKING FRONT LEFT!" );
  if ( IR_FR.distance() < FORWARD_SAFE_DISTANCE )
    Serial.println( "SAFTEY SHUTDOWN ERROR! OBSTACLE IS BLOCKING FRONT RIGHT!" );

  //BACKWARD SAFTEY CHECKS
  if ( IR_BL.distance() < BACKWARD_SAFE_DISTANCE )
    Serial.println( "SAFTEY SHUTDOWN ERROR! OBSTACLE IS BLOCKING BACK LEFT!" );
  if ( IR_BR.distance() < BACKWARD_SAFE_DISTANCE )
    Serial.println( "SAFTEY SHUTDOWN ERROR! OBSTACLE IS BLOCKING BACK RIGHT!" );
  
  
  return 1;
}


void keepAlive_SAFTEYSTOP()
{
  keepAlive_1--;
    if (keepAlive_1 <= 0)
    {
      //MAKE SURE TO SET MOTORS BACK TO ZERO AFTER THE TIMEOUT IN CASE OF FAILURE AT FULL SPEED
      keepAlive_1 = KEEP_ALIVE_TIMEOUT_1;
      keepAlive_2--;

      //Serial.print("blip - ");
      //irReadOut();
      //Serial.print(elapsed);
      //Serial.print(" milliseconds elapsed --- ");
      
      //Serial.println(keepAlive_2);
      irReadOut_SAFTEYSTOP();
    }
    if (keepAlive_2 <= 0)
    {
      finished=millis();
      elapsed=finished-start;
      start=millis();
      Serial.print("KEEPALIVE HAS EXPIRED!!! SHUTTING DOWN CONNECTION NOW -- ");
      Serial.print(elapsed);
      Serial.println(" milliseconds");
      keepAlive_2 = KEEP_ALIVE_TIMEOUT_2;
      
      setMotorCommand(+0000, +0000, +0000, +0000);
    }
}

void setMotorCommand(int target_Motor1_Command, int target_Motor2_Command, int target_FlipperMotor_Command, int target_LACT_Command)
{
  Serial.println("--------- NEW TARGET Motor Values -----------");
  Serial.print("target_Motor1_Command: ");
  Serial.println(target_Motor1_Command);
  Serial.print("target_Motor2_Command: ");
  Serial.println(target_Motor2_Command);      
  Serial.print("target_FlipperMotor_Command: ");
  Serial.println(target_FlipperMotor_Command);      
  Serial.print("target_LACT_Command: ");
  Serial.println(target_LACT_Command);      
  Serial.println("---------------------------------------");
}

void resetKeepAlive()
{
  keepAlive_1 = KEEP_ALIVE_TIMEOUT_1;
  keepAlive_2 = KEEP_ALIVE_TIMEOUT_2;
}

